/*
 * Multi precision toolbox for Scilab
 * Copyright (C) 2009 - Matthew Arsenault
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.    The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 */

#include "mps_priv.h"
#include "mps_assert.h"

/* Raise each element in op1 to the power op2 */
int mps_pow( const mps_ptr rop, const mps_ptr op1, const mpfr_ptr op2, const mpfr_rnd_t rnd )
{
    mpfr_ptr subrop, subop1;
    unsigned int i;
    const unsigned int matsize = MPS_SIZE(op1);

    MPS_ASSERT_MSG( MPS_SAME_SIZE(rop,op1),
        "Result operand differ in dimension in mps_pow()\n" );

    for ( i = 1; i <= matsize; ++i )
    {
        subrop = mps_get_ele_col( rop, i );
        subop1 = mps_get_ele_col( op1, i );

        mpfr_pow( subrop, subop1, op2, rnd );
    }

    return 0;
}

/* Raise each element in op1 to the power op2 */
int mps_pow_ui_mpfr( const mps_ptr rop, const mps_ptr op1, const unsigned int op2, const mpfr_rnd_t rnd )
{
    mpfr_ptr subrop, subop1;
    unsigned int i;
    const unsigned int matsize = MPS_SIZE(op1);

    MPS_ASSERT_MSG( MPS_SAME_SIZE(rop,op1),
        "Result operand differ in dimension in mps_pow_ui_mpfr()\n" );

    for ( i = 1; i <= matsize; ++i )
    {
        subrop = mps_get_ele_col( rop, i );
        subop1 = mps_get_ele_col( op1, i );

        mpfr_pow_ui( subrop, subop1, op2, rnd );
    }

    return 0;
}

/* Raise each element in op1 to the power op2 */
int mps_ui_pow( const mps_ptr rop, const unsigned int op1, const mps_ptr op2, const mpfr_rnd_t rnd )
{
    mpfr_ptr subrop, subop2;
    unsigned int i;
    const unsigned int matsize = MPS_SIZE(op2);

    MPS_ASSERT_MSG( MPS_SAME_SIZE(rop,op2),
        "Result operand differ in dimension in mps_ui_pow()\n" );

    for ( i = 1; i <= matsize; ++i )
    {
        subrop = mps_get_ele_col( rop, i );
        subop2 = mps_get_ele_col( op2, i );

        mpfr_ui_pow( subrop, op1, subop2, rnd );
    }

    return 0;
}

/* Raise each element in op1 to the power op2 */
int mps_pow_si( const mps_ptr rop, const mps_ptr op1, const long int op2, const mpfr_rnd_t rnd )
{
    mpfr_ptr subrop, subop1;
    unsigned int i;
    const unsigned int matsize = MPS_SIZE(op1);

    MPS_ASSERT_MSG( MPS_SAME_SIZE(rop,op1),
        "Result operand differ in dimension in mps_pow_si()\n" );

    for ( i = 1; i <= matsize; ++i )
    {
        subrop = mps_get_ele_col( rop, i );
        subop1 = mps_get_ele_col( op1, i );

        mpfr_pow_si( subrop, subop1, op2, rnd );
    }

    return 0;
}

