/*
 * Multi precision toolbox for Scilab
 * Copyright (C) 2011 - Jonathan Blanchard
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 */

#include "MALLOC.h"
#include "sciprint.h"
#include "stack-c.h"
#include "api_common.h"
#include "api_double.h"
#include "api_string.h"

#include "scimps.h"

int sci_mps_log( char *fname, unsigned long fname_len )
{
    int *arg1, *arg2 = NULL, *arg3 = NULL;
    int m2, n2;
    double *dptr2;
    mp_rnd_t rnd = GMP_RNDN;
    int *rnd_arg = NULL;
    int argnum = 0;
    int rnum = 0;
    int typearg1, typearg2, typearg3;
    mps_ptr rop, op;

    /* Check the number of lhs and rhs arguments. */
    CheckRhs(1,3);
    CheckLhs(1,1);

    /* Get address of the first Rhs argument. */
    getVarAddressFromPosition( pvApiCtx, 1, &arg1 );
    getVarType( pvApiCtx, arg1, &typearg1 );
    
    if( typearg1 != sci_mlist )
    {
        sciprint("%s: Wrong type for argument 1. Multi-precision matrix expected.\n",fname);
        return 0;
    }

    rop = MpsGetMatrix( 1 );

    if( Rhs == 1 )
    {
        goto TLIST;
    }
    else if( Rhs == 2)
    {
        getVarAddressFromPosition( pvApiCtx, 2, &arg2 );
        getVarType( pvApiCtx, arg2, &typearg2 );
        if( typearg2 == sci_mlist )
            goto TLIST_TLIST;
        else if( typearg2 == sci_matrix )
            goto TLIST_DOUBLE;
        else if( typearg2 == sci_strings )
        {
            rnd_arg = arg2;
            argnum = 2;
            goto TLIST;
        }
        else
        {
            sciprint("%s: Wrong type for argument 2. Multi-precision or Scilab matrix or single character string expected.\n",fname);
            return 0;
        }
    }
    else
    {
        getVarAddressFromPosition( pvApiCtx, 2, &arg2 );
        getVarAddressFromPosition( pvApiCtx, 3, &arg3 );
        getVarType( pvApiCtx, arg2, &typearg2 );
        getVarType( pvApiCtx, arg3, &typearg3 );

        if( typearg3 == sci_strings )
        {
            argnum = 3;
            rnd_arg = arg3;
        }
        else
        {
            sciprint("%s: Wrong type for argument 3. Single character string expected.\n",fname);
            return 0;
        }

        if( typearg2 == sci_mlist )
            goto TLIST_TLIST;
        else if( typearg2 == sci_matrix )
            goto TLIST_DOUBLE;
        else
        {
            sciprint("%s: Wrong type for argument 2. Multi-precision or Scilab matrix expected.\n",fname);
            return 0;
        }
    }

    TLIST:

        if( rnd_arg != 0 )
        {
            rnum = MpsGetRoundingArg( fname, &rnd, rnd_arg, argnum );
            if( rnum != 0 )
                return 0;
        }

        mps_log( rop, rop, rnd );

    goto END;


    TLIST_TLIST:

        if( rnd_arg != NULL )
        {
            rnum = MpsGetRoundingArg( fname, &rnd, rnd_arg, argnum );
            if( rnum != 0 )
                return 0;
        }

        op = MpsGetMatrix( 2 );
        if( (MPS_NUMROW(rop) != MPS_NUMROW(op)) || (MPS_NUMCOL(rop) != MPS_NUMCOL(op)) )
        {
            sciprint("%s: Inconsistent matrix operation. Input and output operands differ in size.\n",fname);
            return 1;
        }
        
        mps_log( rop, op, rnd );

    goto END;


    TLIST_DOUBLE:

        if( rnd_arg != NULL )
        {
            rnum = MpsGetRoundingArg( fname, &rnd, rnd_arg, argnum );
            if( rnum != 0 )
                return 0;
        }

        getMatrixOfDouble( pvApiCtx, arg2, &m2, &n2, &dptr2 );

        if( (MPS_NUMROW(rop) != m2) || (MPS_NUMCOL(rop) != n2) )
        {
            sciprint("%s: Inconsistent matrix operation. Input and output operands differ in size.\n",fname);
            return 1;
        }

        mps_log_double( rop, dptr2, MPS_COL_ORDER, rnd );

    goto END;

    END:

    return 0;
}
