/*
 * Multi precision toolbox for Scilab
 * Copyright (C) 2009 - Jonathan Blanchard
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 */

#include "MALLOC.h"
#include "sciprint.h"
#include "stack-c.h"
#include "api_common.h"
#include "api_double.h"
#include "api_string.h"

#include "scimps.h"

/*
 * Matrix multiplication.
 * Scilab definition :
 * mps_mul( rop, op1, op2 )
 * rop : Result upper trianlge matrix.
 * op1, op2 : Input matrices, Cannot be the same as the output matrix.
 */
int sci_mps_mat_mul( char *fname, unsigned long fname_len )
{
    int *arg1, *arg2, *arg3;
    int m1, m2, n1, n2;
    int typearg2, typearg3;
    mps_ptr rop, op1, op2;
    double *dptr1, *dptr2;

    /* Check the number of lhs and rhs arguments. */
    CheckRhs(3,3);
    CheckLhs(1,1);

    /* Get address of inputs */
    getVarAddressFromPosition( pvApiCtx, 1, &arg1 );
    getVarAddressFromPosition( pvApiCtx, 2, &arg2 );
    getVarAddressFromPosition( pvApiCtx, 3, &arg3 );

    /* Check the size and type of the arguments. */
    if( MpsCheck3( fname, arg1, arg2, arg3 ) != 0 )
        return 0;

    /* Fetch the result operand. */
    rop = MpsGetMatrix( 1 );

    getVarType( pvApiCtx, arg2, &typearg2 );
    getVarType( pvApiCtx, arg3, &typearg3 );

    /* Find which situation we are in and jump to the corresponding case. */
    switch (typearg2)
    {
        case sci_matrix:
            switch (typearg3)
            {
                case sci_matrix:
                    goto DOUBLE_DOUBLE;
                case sci_mlist:
                    goto DOUBLE_TLIST;
                default:
                    return 0;
    		}
        case sci_mlist:
            switch (typearg3)
            {
                case sci_matrix:
                    goto TLIST_DOUBLE;
                case sci_mlist:
                    goto TLIST_TLIST;
                default:
                    return 0;
            }
        default:
            return 0;
    }

    /* Never place something here...*/

    DOUBLE_DOUBLE: /* (double*, double*) */

        getMatrixOfDouble( pvApiCtx, arg2, &m1, &n1, &dptr1 );
        getMatrixOfDouble( pvApiCtx, arg3, &m2, &n2, &dptr2 );

          sciprint("%s: Matrix multiplication not supported between two matrix of double.\n",fname);
          return 0;

    goto END;

    DOUBLE_TLIST: /* (double*, mps_t) */

        getMatrixOfDouble( pvApiCtx, arg2, &m1, &n1, &dptr1 );
        op2 = MpsGetMatrix( 3 );

        mps_double_mat_mul( rop, dptr1, m1, 0, op2, GMP_RNDN );

    goto END;

    TLIST_DOUBLE: /* (mps_t, double*) */

        op1 = MpsGetMatrix( 2 );
        getMatrixOfDouble( pvApiCtx, arg3, &m2, &n2, &dptr2 );

        mps_mat_mul_double( rop, op1, dptr2, n2, 0, GMP_RNDN );

    goto END;

    TLIST_TLIST: /* (mps_t, mps_t) */

        op1 = MpsGetMatrix( 2 );
        op2 = MpsGetMatrix( 3 );

        mps_mat_mul( rop, op1, op2, GMP_RNDN );

    END:

    return 0;
}
