/*
 * Multi precision toolbox for Scilab
 * Copyright (C) 2011 - Jonathan Blanchard
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 */

#include "MALLOC.h"
#include "sciprint.h"
#include "stack-c.h"
#include "api_common.h"
#include "api_double.h"
#include "api_string.h"

#include "scimps.h"

/*
 * Matrix transposition.
 * Scilab definition :
 * mps_transpose( rop, op )
 * rop : Result operand, multi-precision matrix.
 * op : Multi-precision matrix to be transposed.
 *
 */
int sci_mps_transpose( char *fname, unsigned long fname_len )
{
    int *arg1, *arg2;
    mps_ptr rop, op;

    /* Check the number of lhs and rhs arguments. */
    CheckRhs(2,2);
    CheckLhs(1,1);

    /* Get address of inputs */
    getVarAddressFromPosition( pvApiCtx, 1, &arg1 );
    getVarAddressFromPosition( pvApiCtx, 2, &arg2 );

//    if( MpsCheck2( fname, arg1, arg2, NULL ) != 0 )
//        return 0;

    /* Fetch the operands. */
    rop = MpsGetMatrix( 1 );
    op = MpsGetMatrix( 2 );

    if( mps_isalias( rop, op ) )
    {
        mps_mat_transpose( rop );
//        sciprint("Result operand cannot be the same as the input operand.",fname);
    }
    else
    {
        mps_full_mat_transpose( rop, op, GMP_RNDN );
    }

    return 0;
}
